<?php

namespace Ulmus\Api;

use Ulmus;

class RequestBuilder implements Ulmus\Query\QueryBuilderInterface
{
    public Ulmus\Query\QueryBuilderInterface $parent;

    public array $parameters = [];

    public array $values = [];

    public string $whereConditionOperator = Ulmus\Query\Where::CONDITION_AND;

    public string $havingConditionOperator = Ulmus\Query\Where::CONDITION_AND;

    protected int $parameterIndex = 0;

    protected array $queryStack = [];

    public function values(array $dataset) : self
    {
        $this->addValues(array_values($dataset));

        return $this;
    }

    public function where(/* stringable*/ $field, $value, string $operator = Ulmus\Query\Where::OPERATOR_EQUAL, string $condition = Ulmus\Query\Where::CONDITION_AND, bool $not = false) : self
    {
        # Empty IN case
        if ( [] === $value ) {
            return $this;
        }

        if ( $this->where ?? false ) {
            $where = $this->where;
        }
        elseif ( null === ( $where = $this->getFragment(RequestBuilder\Filter::class) ) ) {
            $this->where = $where = new RequestBuilder\Filter($this);
            $this->push($where);
        }

        $this->whereConditionOperator = $operator;

        $where->add($field, $value, $operator, $condition, $not);

        return $this;
    }

    public function notWhere($field, $value, string $operator = Ulmus\Query\Where::CONDITION_AND) : self
    {
        return $this->where($field, $value, $operator, true);
    }

    public function limit(int $value) : self
    {
        /*if ( null === $limit = $this->getFragment(Ulmus\Query\Limit::class) ) {
            $limit = new Query\Limit();
            $this->push($limit);
        }

        $limit->set($value);
*/
        return $this;
    }

    public function offset(int $value) : self
    {
        if ( null === $offset = $this->getFragment(Ulmus\Query\Offset::class) ) {
            $offset = new Ulmus\Query\Offset();
            $this->push($offset);
        }

        $offset->set($value);

        return $this;
    }

    public function orderBy(string $field, ? string $direction = null) : self
    {
        return $this;
    }

    public function push(Ulmus\Query\Fragment $queryFragment) : self
    {
        $this->queryStack[] = $queryFragment;

        return $this;
    }

    public function pull(Ulmus\Query\Fragment $queryFragment) : self
    {
        return array_shift($this->queryStack);
    }

    public function render(bool $skipToken = false) : array
    {
        $stack = [];

        foreach($this->queryStack as $fragment) {
            $stack = array_merge($stack, (array) $fragment->render($skipToken));
        }

        return $stack;
    }

    public function reset() : void
    {
        $this->parameters = $this->values = $this->queryStack = [];
        $this->whereConditionOperator = Ulmus\Query\Where::CONDITION_AND;
        $this->parameterIndex = 0;

        unset($this->where);
    }

    public function getFragment(string $class, int $index = 0) : ? Ulmus\Query\Fragment
    {
        foreach($this->queryStack as $item) {
            if ( get_class($item) === $class ) {
                if ( $index-- === 0 ) {
                    return $item;
                }
            }
        }

        return null;
    }

    public function removeFragment(\Ulmus\Query\Fragment|array|\Stringable|string $fragment) : void
    {
        foreach($this->queryStack as $key => $item) {
            if ( $item === $fragment ) {
                unset($this->queryStack[$key]);
            }
        }
    }

    public function __toString() : string
    {
        return $this->render();
    }

    public function addParameter($value, string $key = null) : string
    {
        if ( $this->parent ?? false ) {
            return $this->parent->addParameter($value, $key);
        }

        if ( $key === null ) {
            $key = ":p" . $this->parameterIndex++;
        }

        $this->parameters[$key] = $value;

        return $key;
    }

    public function addValues(array $values) : void
    {
        $this->values = $values;
    }
}
